#!/usr/bin/env python
# encoding: utf-8

import pyfits
import numpy

import sys

def main():
    """
    csv-to-fits.py  INPUT.csv  OUTPUT.fits

    CSV ファイルを FITS bintable に変換します。

    - CSV の第 1 行は、列の名前です。
        - id という名の列は、64ビット整数として扱われます。
        - starnotgal という名の列は、bool 値 (0/1) として扱われます。
        - ra, dec という名の列は、 倍精度浮動小数点数として扱われます。
        - それ以外の名の列は、単精度浮動小数点数となります。
    """

    if len(sys.argv) != 3:
        raise Exception(
            "commandline: {}  INPUT.csv  OUTPUT.fits"
            .format(sys.argv[0]))

    recarray = loadCsvAsNumpy(sys.argv[1])
    bintable = convertNumpyToFitsBinTable(recarray)
    saveFitsBinTable(bintable, sys.argv[2])


def loadCsvAsNumpy(filename):
    dtypes = {
        "id" : numpy.int64,
        "starnotgal" : numpy.bool,
        "RA" : numpy.float64,
        "DEC" : numpy.float64,
    }

    def nameFilter(name):
        uname = name.upper()
        if uname == "RA" or uname == "DEC":
            return uname
        elif uname == "RA_ERR":
            return "RA_err"
        elif uname == "DEC_ERR":
            return "DEC_err"
        elif uname == "ID" or uname == "STARNOTGAL":
            return name.lower()
        else:
            return name

    with open(filename) as f:
        headers = map(nameFilter, f.readline().strip().split(','))

    dtype = [ (name, dtypes[name] if name in dtypes else numpy.float32)
        for name in headers ]

    return numpy.loadtxt(filename,
        dtype = dtype, delimiter = ',', skiprows = 1)


def convertNumpyToFitsBinTable(recarray):
    return pyfits.FITS_rec.from_columns(pyfits.ColDefs(recarray))


def saveFitsBinTable(bintable, filename):
    primaryHDU = pyfits.PrimaryHDU()
    binTableHDU = pyfits.BinTableHDU(bintable)
    pyfits.HDUList([primaryHDU, binTableHDU]).writeto(filename)


if __name__ == "__main__":
    main()
